@@ -28,30 +28,75 @@
 
 using namespace KScreen;
 
-class Config::Private
+class Config::Private : public QObject
 {
-  public:
-    Private():
-      valid(true)
+    Q_OBJECT
+public:
+    Private(Config *parent)
+        : QObject(parent)
+        , valid(true)
+        , q(parent)
     { }
 
-    Private(const Private &other):
-      valid(other.valid),
-      primaryOutput(other.primaryOutput)
+    KScreen::OutputPtr findPrimaryOutput() const
     {
-      screen = other.screen->clone();
-      Q_FOREACH (const OutputPtr &otherOutput, other.outputs) {
-          outputs.insert(otherOutput->id(), otherOutput->clone());
-      }
+        auto iter = std::find_if(outputs.constBegin(), outputs.constEnd(),
+                                 [](const KScreen::OutputPtr &output) -> bool {
+                                    return output->isPrimary();
+                                 });
+        return iter == outputs.constEnd() ? KScreen::OutputPtr() : iter.value();
+    }
+
+    void onPrimaryOutputChanged()
+    {
+        const KScreen::OutputPtr output(qobject_cast<KScreen::Output*>(sender()), [](void *) {});
+        Q_ASSERT(output);
+        if (output->isPrimary()) {
+            q->setPrimaryOutput(output);
+        } else {
+            q->setPrimaryOutput(findPrimaryOutput());
+        }
+    }
+
+    OutputList::Iterator removeOutput(OutputList::Iterator iter)
+    {
+        if (iter == outputs.end()) {
+            return iter;
+        }
+
+        OutputPtr output = iter.value();
+        if (!output) {
+            return outputs.erase(iter);
+        }
+
+        const int outputId = iter.key();
+        iter = outputs.erase(iter);
+
+        if (primaryOutput == output) {
+            q->setPrimaryOutput(OutputPtr());
+        }
+        output->disconnect(q);
+
+        Q_EMIT q->outputRemoved(outputId);
+
+        return iter;
     }
 
     bool valid;
     ScreenPtr screen;
     OutputPtr primaryOutput;
     OutputList outputs;
+
+private:
+    Config *q;
 };
 
 bool Config::canBeApplied(const ConfigPtr &config)
+{
+    return canBeApplied(config, ValidityFlag::None);
+}
+
+bool Config::canBeApplied(const ConfigPtr &config, ValidityFlags flags)
 {
     ConfigPtr currentConfig = BackendManager::instance()->config();
     if (!currentConfig) {
@@ -93,7 +138,6 @@ bool Config::canBeApplied(const ConfigPtr &config)
             return false;
         }
 
-
         const ModePtr currentMode = output->currentMode();
 
         const QSize outputSize = currentMode->size();
@@ -124,6 +168,11 @@ bool Config::canBeApplied(const ConfigPtr &config)
         }
     }
 
+    if (flags & ValidityFlag::RequireAtLeastOneEnabledScreen && enabledOutputsCount == 0) {
+        qCDebug(KSCREEN) << "canBeAppled: There are no enabled screens, at least one required";
+        return false;
+    }
+
     const int maxEnabledOutputsCount = config->screen()->maxActiveOutputsCount();
     if (enabledOutputsCount > maxEnabledOutputsCount) {
         qCDebug(KSCREEN) << "canBeApplied: Too many active screens. Requested: " << enabledOutputsCount << ", Max: " << maxEnabledOutputsCount;
@@ -144,13 +193,7 @@ bool Config::canBeApplied(const ConfigPtr &config)
 
 Config::Config()
  : QObject(0)
- , d(new Private())
-{
-}
-
-Config::Config(Config::Private *dd)
-  : QObject()
-  , d(dd)
+ , d(new Private(this))
 {
 }
 
@@ -162,7 +205,14 @@ Config::~Config()
 
 ConfigPtr Config::clone() const
 {
-    return ConfigPtr(new Config(new Private(*d)));
+    ConfigPtr newConfig(new Config());
+    newConfig->d->screen = d->screen->clone();
+    for (const OutputPtr &ourOutput : d->outputs) {
+        newConfig->addOutput(ourOutput->clone());
+    }
+    newConfig->d->primaryOutput = newConfig->d->findPrimaryOutput();
+
+    return newConfig;
 }
 
 
@@ -178,11 +228,7 @@ void Config::setScreen(const ScreenPtr &screen)
 
 OutputPtr Config::output(int outputId) const
 {
-    if (!d->outputs.contains(outputId)) {
-        return OutputPtr();
-    }
-
-    return d->outputs[outputId];
+    return d->outputs.value(outputId);
 }
 
 OutputList Config::outputs() const
@@ -209,52 +255,62 @@ OutputPtr Config::primaryOutput() const
         return d->primaryOutput;
     }
 
-    Q_FOREACH(const OutputPtr &output, d->outputs) {
-        if (output->isPrimary()) {
-            d->primaryOutput = output;
-            return d->primaryOutput;
-        }
-    }
-
-    return OutputPtr();
+    d->primaryOutput = d->findPrimaryOutput();
+    return d->primaryOutput;
 }
 
-void Config::setPrimaryOutput(const OutputPtr &output)
+void Config::setPrimaryOutput(const OutputPtr &newPrimary)
 {
-    if (primaryOutput() == output) {
+    // Don't call primaryOutput(): at this point d->primaryOutput is either
+    // initialized, or we need to look for the primary anyway
+    if (d->primaryOutput == newPrimary) {
         return;
     }
 
-    qDebug(KSCREEN) << "Primary output changed from" << d->primaryOutput
-                    << "(" << (d->primaryOutput.isNull() ? "none" : d->primaryOutput->name()) << ") to"
-                    << output << "(" << (output.isNull() ? "none" : output->name()) << ")";
-    d->primaryOutput = output;
+    qDebug(KSCREEN) << "Primary output changed from" << primaryOutput()
+                    << "(" << (primaryOutput().isNull() ? "none" : primaryOutput()->name()) << ") to"
+                    << newPrimary << "(" << (newPrimary.isNull() ? "none" : newPrimary->name()) << ")";
 
-    Q_EMIT primaryOutputChanged(output);
+    for (OutputPtr &output : d->outputs) {
+        disconnect(output.data(), &KScreen::Output::isPrimaryChanged,
+                d, &KScreen::Config::Private::onPrimaryOutputChanged);
+        output->setPrimary(output == newPrimary);
+        connect(output.data(), &KScreen::Output::isPrimaryChanged,
+                d, &KScreen::Config::Private::onPrimaryOutputChanged);
+    }
+
+    d->primaryOutput = newPrimary;
+    Q_EMIT primaryOutputChanged(newPrimary);
 }
 
 void Config::addOutput(const OutputPtr &output)
 {
     d->outputs.insert(output->id(), output);
+    connect(output.data(), &KScreen::Output::isPrimaryChanged,
+            d, &KScreen::Config::Private::onPrimaryOutputChanged);
 
     Q_EMIT outputAdded(output);
+
+    if (output->isPrimary()) {
+        setPrimaryOutput(output);
+    }
 }
 
 void Config::removeOutput(int outputId)
 {
-    OutputPtr output = d->outputs.take(outputId);
-    if (output) {
-        if (d->primaryOutput == output) {
-            setPrimaryOutput(OutputPtr());
-        }
-    }
-
-    Q_EMIT outputRemoved(outputId);
+    d->removeOutput(d->outputs.find(outputId));
 }
 
 void Config::setOutputs(OutputList outputs)
 {
-    d->outputs = outputs;
+    for (auto iter = d->outputs.begin(), end = d->outputs.end(); iter != end; ) {
+        iter = d->removeOutput(iter);
+        end = d->outputs.end();
+    }
+
+    for (const OutputPtr &output : outputs) {
+        addOutput(output);
+    }
 }
 
 bool Config::isValid() const
@@ -288,19 +344,8 @@ void Config::apply(const ConfigPtr& other)
         }
     }
 
-    // Update primary output
-    bool matched = false;
-    Q_FOREACH (const OutputPtr &output, d->outputs) {
-        if (output->isPrimary()) {
-            setPrimaryOutput(output);
-            matched = true;
-            break;
-        }
-    }
-    if (!matched) {
-        setPrimaryOutput(OutputPtr());
-    }
-
     // Update validity
     setValid(other->isValid());
 }
+
+#include "config.moc"
\ No newline at end of file
